/*
 * SoundClock.c
 *
 */ 

// Define CPU Clock which is used in following header files
#define	F_CPU			8000000UL

// Header Files
#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/eeprom.h>
#include	<util/delay.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

/******************************************************************************
	Decelerations
******************************************************************************/
char recvUSART(char *data);
void sendUSART(char data);
void printStr(char *str);

/******************************************************************************
	GPIO
******************************************************************************/
void initGPIO()
{
	// Global Pull Up Enable for All Input Pins
	MCUCR &= ~0x10;

	// PORT DIR ( 0:Input / 1:Output)
	DDRC  = 0b00000000;
	DDRB  = 0b00000000;
	DDRD  = 0b00000100;

	// PULLUP for INPUT (1:PullUp)
	PORTB = 0b11111111;
	PORTC = 0b11111111;
	PORTD = 0b11111111;
}

void soundEnable(void)
{
	PORTD |= 0x04;
}

void soundSleep(void)
{
	PORTD &=~0x04;
}

char readKey(void)
{
	static char prevKey = 0;
	static char pressCounter1 = 0;
	static char pressCounter2 = 0;
	static char pressCounter3 = 0;
	char key, diff;
	char ret = 0;

	key = ~PINC & 0x0F;
	diff = key ^ prevKey;

	// Key1 Press or Long Press
	if( (diff&0x01) && (key&0x01) ) pressCounter1 = 1;
	if( key&0x01 ){
		if(pressCounter1 >  0 ) pressCounter1++;
		if(pressCounter1 > 16){
			ret |= 0x10;
			pressCounter1 = 0;
		}
	}
	else{
		if(pressCounter1 > 0){
			ret |=0x01;
			pressCounter1 = 0;
		}
	}

	// Key 2
	if( key&0x02 ) pressCounter2 ++;
	else pressCounter2 = 0;
	if( pressCounter2 == 1 || (pressCounter2 > 8 && (pressCounter2 & 0x01)) ) ret |= 0x02;

	// Key 3
	if( key&0x04 ) pressCounter3 ++;
	else pressCounter3 = 0;
	if( pressCounter3 == 1 || (pressCounter3 > 8 && (pressCounter3 & 0x01)) ) ret |= 0x04;

	// Key4
	if( (diff&0x08) && (key&0x08) ) ret |= 0x08;

	prevKey = key;

	return ret;
}

/******************************************************************************
	Timer Function
******************************************************************************/
#define COUNT_INTERVAL	50

volatile unsigned char timerCounter2ms = 0;
volatile unsigned char timerCounter100ms = 0;

ISR(TIMER0_COMPA_vect)
{
	if(timerCounter2ms < COUNT_INTERVAL-1){
		timerCounter2ms++;
	}
	else{
		timerCounter2ms = 0;
		timerCounter100ms++;
	}
}

void initTimer(void)
{
	cli();	// Disable Interrupt

	// TCCR0A ... COM0A1 COM0A0 COM0B1 COM0B0 -      -      WGM01  WGM00
	// TCCR0B ... FC0A   FOC0B  -      -      WGM02  CS02   CS01   CS00
	// 
	// WGM0[2:0]  = 010 (CTC Mode)
	// COM0A[1:0] = 00
	// COM0B[1:0] = 00
	// CS0[2:0]   = 011 // Clock Select = Clock/64
	// FOC0A      = 0	// Force Output Compare A
	// FOC0B      = 0	// Force Output Compare B

	TCCR0A = 0b00000010;
	TCCR0B = 0b00000011;		// Clock/64
	OCR0A  = F_CPU/64/500;		// 2msƂɊ荞(500Hz)
	TIMSK0 = 0b0000010;			// rAv荞ݗL
	sei();	// Enable Interrupt
}

/******************************************************************************
	I2C Functions
******************************************************************************/
#define I2C_READ				1
#define I2C_WRITE				0

void initTWI(void)
{
	// Use 200kHz
	// Freq = CPU clock / (16 + 2 * prescaler * TWBR)
	// TWBR = (CPU clock / Freq - 16) / 2 / Prescaler
	TWSR = 0x00;	// TWPS[1:0] = 00 (Prescaler Value = 1)
	TWBR = 0x0C;	// TWI baud rate

	// Enable TWI
	TWCR = (1<<TWEN);
}

void sendTWIStart(void)
{
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
	while ((TWCR & (1<<TWINT)) == 0);
}

void sendTWIStop(void)
{
	TWCR = (1<<TWINT)|(1<<TWSTO)|(1<<TWEN);
}

void writeTWI(unsigned char data)
{
	TWDR = data;
	TWCR = (1<<TWINT)|(1<<TWEN);
	while ((TWCR & (1<<TWINT)) == 0);
}

unsigned char readTWIACK(void)
{
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWEA);
	while ((TWCR & (1<<TWINT)) == 0);
	return TWDR;
}

unsigned char readTWINACK(void)
{
	TWCR = (1<<TWINT)|(1<<TWEN);
	while ((TWCR & (1<<TWINT)) == 0);
	return TWDR;
}

unsigned char getTWIStatus(void)
{
	unsigned char status;

	status = TWSR & 0xF8;
	return status;
}

/******************************************************************************
	LCD Control Functions
******************************************************************************/
#define LCD_I2C_ADDRESS	0x7C
#define LCD_CO			0x80
#define LCD_RS			0x40

void ctrlLCD(char cmd)
{
	sendTWIStart();
	writeTWI( LCD_I2C_ADDRESS | I2C_WRITE );
	writeTWI( 0x00 );	// Control Byte
	writeTWI( cmd );	// Data Byte
	sendTWIStop();
	_delay_us(30);
}

void initLCD(void)
{
	_delay_ms(40);

	ctrlLCD(0x38);
	ctrlLCD(0x39);
	ctrlLCD(0x14);
	ctrlLCD(0x70);
	ctrlLCD(0x56);
	ctrlLCD(0x6C);

	_delay_ms(200);

	ctrlLCD(0x38);
	ctrlLCD(0x0C);
	ctrlLCD(0x01);
	
	_delay_ms(1);
}

void writeLCD(char x, char y, char *data, char len)
{
	char i;
	char *c;

	// Set DDRAM ADDRESS 0
	ctrlLCD( 0x80 + y * 0x40 + x );

	sendTWIStart();
	writeTWI( LCD_I2C_ADDRESS | I2C_WRITE );

	c = data;
	for(i=0; i<len-1; i++){
		writeTWI( LCD_CO | LCD_RS );	// Control Byte
		writeTWI( *c++ );				// Data Byte
		_delay_us(30);
	}
	writeTWI( LCD_RS );
	writeTWI( *c );
	sendTWIStop();
	_delay_us(30);
}

/******************************************************************************
	RTC Control Functions
******************************************************************************/
#define RTC_I2C_ADDRESS	0xA2
#define DEC2BCD(n)		(((n/10)<<4) | (n%10))

typedef struct { 
	char sec;
	char min;
	char hour;
	char alarmEnable;
} TIME_DATA;

typedef struct {
	char day;
	char month;
	char year;
	char weekDay;
} DATE_DATA;

TIME_DATA	g_RtcTimeData;
DATE_DATA	g_RtcDateData;

char g_SignalMode = 0;

void readRtcTime(TIME_DATA *dat)
{
	char buff;

	sendTWIStart();
	writeTWI( RTC_I2C_ADDRESS | I2C_WRITE );
	writeTWI( 0x02 );	// ADDRESS

	sendTWIStart();
	writeTWI( RTC_I2C_ADDRESS | I2C_READ );

	buff = readTWIACK();
	dat->sec    = ((buff>>4)&0x07)*10 + (buff&0x0F);

	buff = readTWIACK();
	dat->min    = ((buff>>4)&0x07)*10 + (buff&0x0F);

	buff = readTWINACK();
	dat->hour   = ((buff>>4)&0x03)*10 + (buff&0x0F);

	sendTWIStop();
}

void readRtcDate(DATE_DATA *dat)
{
	char buff;

	sendTWIStart();
	writeTWI( RTC_I2C_ADDRESS | I2C_WRITE );
	writeTWI( 0x05 );	// ADDRESS

	sendTWIStart();
	writeTWI( RTC_I2C_ADDRESS | I2C_READ );

	buff = readTWIACK();
	dat->day    = ((buff>>4)&0x03)*10 + (buff&0x0F);

	buff = readTWIACK();
	dat->weekDay=                       (buff&0x07);

	buff = readTWIACK();
	dat->month  = ((buff>>4)&0x01)*10 + (buff&0x0F);

	buff = readTWINACK();
	dat->year   = ((buff>>4)&0x0F)*10 + (buff&0x0F);

	sendTWIStop();
}

void writeRtcTime(char hour, char min, char sec)
{
	char buff;

	sendTWIStart();
	writeTWI( RTC_I2C_ADDRESS | I2C_WRITE );
	writeTWI( 0x02 );					// Write Address

	buff = DEC2BCD(sec);
	writeTWI( buff );
	buff = DEC2BCD(min);
	writeTWI( buff );
	buff = DEC2BCD(hour);
	writeTWI( buff );

	sendTWIStop();
}

void writeRtcDate(char year, char month, char day)
{
	char buff;
	char weekDay;

	int y, m, d;
	y = year + 2000;
	m = month;
	d = day;
	if(m < 3){
		y--;
		m+=12;
	}
	weekDay = (y + y/4 - y/100 + y/400 + (13*m+8)/5 + d) % 7;

	sendTWIStart();
	writeTWI( RTC_I2C_ADDRESS | I2C_WRITE );
	writeTWI( 0x05 );					// Write Address

	buff = DEC2BCD(day);
	writeTWI( buff );
	buff = DEC2BCD(weekDay);
	writeTWI( buff );
	buff = DEC2BCD(month);
	writeTWI( buff );
	buff = DEC2BCD(year);
	writeTWI( buff );

	sendTWIStop();
}

/******************************************************************************
	EEPROM Parameter Save/Restore Function
******************************************************************************/
TIME_DATA g_AlarmInfo[4];

void saveAlarmInfo(void)
{
	eeprom_busy_wait();
	eeprom_write_block(g_AlarmInfo, 0x00, sizeof(TIME_DATA) * 4);
	eeprom_busy_wait();
}

void loadAlarmInfo(void)
{
	eeprom_busy_wait();
	eeprom_read_block(g_AlarmInfo, 0x00, sizeof(TIME_DATA) * 4);
	eeprom_busy_wait();
}

/******************************************************************************
	FIFO Functions
******************************************************************************/
typedef struct {
	int wp;
	int rp;
	int size;
	char *buff;
} FiFo;

void initFiFo(FiFo *fifo, char *buff, int size)
{
	fifo->wp = 0;
	fifo->rp = 0;
	fifo->buff = buff;
	fifo->size = size;
}

void writeFiFo(FiFo *fifo, char data)
{
	char wp;
	wp = fifo->wp + 1;
	if(wp == fifo->size) wp = 0;
	if(wp != fifo->rp){
		fifo->buff[fifo->wp] = data;
		fifo->wp = wp;
	}
}

char readFiFo(FiFo *fifo, char *data)
{
	if(fifo->rp != fifo->wp){
		*data = fifo->buff[fifo->rp];
		if(fifo->rp + 1 == fifo->size) fifo->rp = 0;
		else fifo->rp++;
		return 1;
	}
	return 0;
}

char isAvailable(FiFo *fifo)
{
	char wp = fifo->wp;
	char rp = fifo->rp;
	if(wp - rp < 0){
		return wp - rp + fifo->size;
	}
	else{
		return wp - rp;
	}
}

/******************************************************************************
	USART Print Functions
******************************************************************************/
#define	PRINT_BUFFER_SIZE	80

FiFo printFiFo;
char printBuff[PRINT_BUFFER_SIZE];
volatile char g_PrintFinished;
volatile char g_VoiceReady;

void initPrint(void)
{
	initFiFo(&printFiFo, printBuff, PRINT_BUFFER_SIZE);
	g_PrintFinished = 1;
	g_VoiceReady = 1;
}

void printStr(char *str)
{
	char data = 0;

	g_VoiceReady = 0;

	while(*str){
		writeFiFo(&printFiFo, *str++);
	}
	if(g_PrintFinished){
		readFiFo(&printFiFo, &data);
		sendUSART(data);
	}
}

/******************************************************************************
	USART Functions
******************************************************************************/
#define BAUDRATE		9600							// Baud rate that we want to use
#define BAUD_PRESCALLER	(((F_CPU/(BAUDRATE*16UL)))-1)	// Register value to set

void initUSART(void)
{
	cli();	// Disable Interrupt
	UBRR0H = (uint8_t)(BAUD_PRESCALLER>>8);
	UBRR0L = (uint8_t)(BAUD_PRESCALLER);
	UCSR0B = (1<<RXEN0) | (1<<TXEN0) | (1<<TXCIE0) | (1<<RXCIE0);	// Enable Rx & Tx, Use Tx & Rx Interrupt
	UCSR0C = (1<<UCSZ00) | (1<<UCSZ01);								// 8bit length
	sei();	// Enable Interrupt
}

void sendUSART(char data)
{
	while(!(UCSR0A & (1<<UDRE0)));
	UDR0 = data;
}

char recvUSART(char *data)
{
	if(!(UCSR0A & (1<<RXC0))){
		return 0;
	}
	else{
		*data = UDR0;
		return 1;
	}
}

ISR(USART_TX_vect)
{
	char data;
	if( readFiFo(&printFiFo, &data) ){
		g_PrintFinished = 0;
		sendUSART(data);
	}
	else{
		g_PrintFinished = 1;
	}
}

ISR(USART_RX_vect)
{
	char data = 0;
	recvUSART(&data);
	if(data == '>'){
		g_VoiceReady = 1;
	}
}

/******************************************************************************
	VOICE CONTROL
******************************************************************************/
char *weekDayVoice[7]={"nichi","getu","ka","sui","moku","kinn","do"};
char VoiceBuff[127];

char voiceCreateAlarm(char *buff, char alarm, char state)
{
	char rc = 0;

	switch(state){
		case 1:
		case 2:
		case 3:
			sprintf(buff, "#J\r" );
			rc = state + 1;
			break;
		case 4:
			sprintf(buff,
				"<NUMK VAL=%d COUNTER=ji><NUMK VAL=%d COUNTER=funn>ni;narimasita\r",
				g_AlarmInfo[alarm].hour,
				g_AlarmInfo[alarm].min );
			rc = 0;
			break;
		default:
			rc = 0;
			break;
	}

	return rc;	
}

char voiceCreateTimeSignal(char *buff, char state)
{
	char rc = 0;

	switch(state){
		case 1:
			if(g_SignalMode == 1){
				sprintf(buff, "<NUMK VAL=%d COUNTER=ji>desu.\r", g_RtcTimeData.hour);
				rc = 0;
			}
			else{
				sprintf(buff, "#K\r" );
				rc = state + 1;
			}
			break;
		case 2:
			sprintf(buff,
				"yukkuriga;<NUMK VAL=%d COUNTER=ji>wo;oshiraseshimashita.\r",
				g_RtcTimeData.hour );
			rc = 0;
			break;
		default:
			rc = 0;
			break;
	}

	return rc;
}

char voiceCreateGreeting(char *buff, char state)
{
	char rc = 0;

	switch(state){
		case 1:
			if( g_RtcDateData.month == 1 && g_RtcDateData.day == 1 ){
				sprintf(buff, "akemashite/omedeto-gozaimasu\r" );
			}
			else if( g_RtcTimeData.hour > 4 && g_RtcTimeData.hour < 12 ){
				sprintf(buff, "ohayo-gozaimasu\r" );
			}
			else if( g_RtcTimeData.hour > 11 && g_RtcTimeData.hour < 19 ){
				sprintf(buff, "konnichiwa\r" );
			}
			else {
				sprintf(buff, "konbanwa\r" );
			}
			rc = 2;
			break;
		case 2:
			sprintf(buff,
				"kyo'-wa/<NUMK VAL=%d COUNTER=gatu><NUMK VAL=%d COUNTER=nichi>;/%syo'-bi\r",
				g_RtcDateData.month, g_RtcDateData.day,
				weekDayVoice[g_RtcDateData.weekDay] );
			rc = 3;
			break;
		case 3:
			sprintf(buff,
				"ji'kokuwa/<NUMK VAL=%d COUNTER=ji>/<NUMK VAL=%d COUNTER=funn>desu.\r",
				g_RtcTimeData.hour, g_RtcTimeData.min );
			rc = 0;
			break;
		default:
			rc = 0;
			break;
	}

	return rc;
}

void voiceControl(char key)
{
	static char voice_state = 0;
	static char type = 0;
	static char alarm = 0;
	char i;

	if( g_VoiceReady == 0 ) return;
	soundSleep();

	// Check Alarm
	for(i=0;i<4;i++){
		if( g_AlarmInfo[i].alarmEnable != 0 &&
			g_RtcTimeData.hour == g_AlarmInfo[i].hour &&
			g_RtcTimeData.min  == g_AlarmInfo[i].min &&
			g_RtcTimeData.sec  == 0 ){
			alarm = i;
			break;
		} 
	}

	// Voice Pattern
	if( voice_state == 0 ){
		if( i < 4 ){
			voice_state = 1;
			type = 1;
		}
		else if( g_RtcTimeData.min  == 0 && g_RtcTimeData.sec  == 0 && g_SignalMode != 0 ){
			voice_state = 1;
			type = 2;
		}
		else if( key & 0x08 ){
			voice_state = 1;
			type = 3;
		}
	}

	// Voice Create & Play
	if(voice_state > 0){
		switch(type){
			case 1: voice_state = voiceCreateAlarm(VoiceBuff, alarm, voice_state); break;
			case 2: voice_state = voiceCreateTimeSignal(VoiceBuff, voice_state); break;
			case 3: voice_state = voiceCreateGreeting(VoiceBuff, voice_state); break;
			default: sprintf(VoiceBuff, "era-.\r"); break;
		}
		_delay_ms(10);
		soundEnable();
		_delay_ms(10);
		printStr("?");
		_delay_ms(10);
		printStr(VoiceBuff);
	}
}

/******************************************************************************
	DISPLAY CONTROL
******************************************************************************/
static char *weekDay[7] = {"SUN","MON","TUE","WED","THU","FRI","SAT"};
static char DispBuffTSet[]  = "TIME SET";
static char DispBuffDSet[]  = "DATE SET";
static char DispBuffSSet[]  = "SIGNAL  ";
char DispBuffDate1[] = "  /     ";
char DispBuffDate2[] = "  /  /  ";
char DispBuffTime1[] = "  :     ";
char DispBuffTime2[] = "  :  :  ";
char DispBuffAlarm[] = "ALARM   ";

void setNum(char *buf, char num)
{
	buf[0] = '0' + num/10;
	buf[1] = '0' + num%10;
}

void setSpc(char *buf)
{
	buf[0] = ' ';
	buf[1] = ' ';
}

void dispDateTime(void)
{
	setNum( &DispBuffDate1[0], g_RtcDateData.month ); 
	setNum( &DispBuffDate1[3], g_RtcDateData.day   );
	writeLCD(0, 0, DispBuffDate1, 5);
	writeLCD(5, 0, weekDay[g_RtcDateData.weekDay], 3);

	setNum( &DispBuffTime2[0], g_RtcTimeData.hour );
	setNum( &DispBuffTime2[3], g_RtcTimeData.min  );
	setNum( &DispBuffTime2[6], g_RtcTimeData.sec  );
	writeLCD(0, 1, DispBuffTime2, 8);
}

void dispDate(void)
{
	setNum( &DispBuffDate2[0], g_RtcDateData.year );
	setNum( &DispBuffDate2[3], g_RtcDateData.month );
	setNum( &DispBuffDate2[6], g_RtcDateData.day   );
	writeLCD(0, 0, DispBuffDSet, 8);
	writeLCD(0, 1, DispBuffDate2, 8);
}

void dispTime(void)
{
	setNum( &DispBuffTime1[0], g_RtcTimeData.hour );
	setNum( &DispBuffTime1[3], g_RtcTimeData.min  );
	writeLCD(0, 0, DispBuffTSet, 8);
	writeLCD(0, 1, DispBuffTime1, 8);
}

void dispAlarm(char n, char key)
{
	setNum( &DispBuffTime1[0], g_AlarmInfo[n].hour );
	setNum( &DispBuffTime1[3], g_AlarmInfo[n].min  );

	if(key & 0x01){
		g_AlarmInfo[n].alarmEnable = ~g_AlarmInfo[n].alarmEnable;
		saveAlarmInfo();
	}

	DispBuffAlarm[5] = '1' + n;
	if( g_AlarmInfo[n].alarmEnable == 0 ){
		DispBuffAlarm[7] = ' ';
	}
	else{
		DispBuffAlarm[7] = '*';
	}

	writeLCD(0, 0, DispBuffAlarm, 8);
	writeLCD(0, 1, DispBuffTime1, 8);
}

void dispSignalMode(char key)
{
	if(key & 0x01){
		g_SignalMode ++;
		if(g_SignalMode>2) g_SignalMode = 0;
	}

	writeLCD(0, 0, DispBuffSSet, 8);

	switch(g_SignalMode){
		case 0: writeLCD(0, 1, "OFF     ", 8); break;
		case 1: writeLCD(0, 1, "SIMPLE  ", 8); break;
		case 2: writeLCD(0, 1, "FULL    ", 8); break;
	}
}

void editDate(char key, char *y, char *m, char *d)
{
	static char blinkCounter = 0;
	static char cursorPos = 0;
	static char dayInMonth[12] = {31,29,31,30,31,30,31,31,30,31,30,31};

	if( key & 0x01 ){
		cursorPos++;
		if(cursorPos>2) cursorPos = 0;
	}
	if( key & 0x02 ) {
		switch(cursorPos){
			case 0: *y=*y+1; if(*y>99) *y=0; break;
			case 1: *m=*m+1; if(*m>12) *m=1; break;
			case 2: *d=*d+1; break;
		}		
	}		
	if( key & 0x04 ) {
		switch(cursorPos){
			case 0: *y=*y-1; if(*y<1) *y=99; break;
			case 1: *m=*m-1; if(*m<1) *m=12; break;
			case 2: *d=*d-1; break;
		}
	}

	if(*d < 1               ) *d = dayInMonth[*m-1]; 
	if(*d > dayInMonth[*m-1]) *d = 1;
 
	blinkCounter++;
	if( cursorPos == 0 && (blinkCounter&0x04) ) setSpc( &DispBuffDate2[0] );
	else setNum( &DispBuffDate2[0], *y );
	if( cursorPos == 1 && (blinkCounter&0x04) ) setSpc( &DispBuffDate2[3] );
	else setNum( &DispBuffDate2[3], *m );
	if( cursorPos == 2 && (blinkCounter&0x04) ) setSpc( &DispBuffDate2[6] );
	else setNum( &DispBuffDate2[6], *d );

	writeLCD(0, 1, DispBuffDate2, 8);
}

void editTime(char key, char *h, char *m)
{
	static char blinkCounter = 0;
	static char cursorPos = 0;

	if( key & 0x01 ){
		cursorPos++;
		if(cursorPos>1) cursorPos = 0;
	}
	if( key & 0x02 ) {
		switch(cursorPos){
			case 0: *h=*h+1; if(*h==24) *h=0; break;
			case 1: *m=*m+1; if(*m==60) *m=0; break;
		}
	}
	if( key & 0x04 ) {
		switch(cursorPos){
			case 0: *h=*h-1; if(*h==0xFF) *h=23; break;
			case 1: *m=*m-1; if(*m==0xFF) *m=59; break;
		}
	}

	blinkCounter++;
	if( cursorPos == 0 && (blinkCounter&0x04) ) setSpc( &DispBuffTime1[0] );
	else setNum( &DispBuffTime1[0], *h );
	if( cursorPos == 1 && (blinkCounter&0x04) ) setSpc( &DispBuffTime1[3] );
	else setNum( &DispBuffTime1[3], *m );

	writeLCD(0, 1, DispBuffTime1, 8);
}

void screenManagement(char key)
{
	static char screen = 0;	// 0:TOP, 1:DATE, 2:TIME, 3:ALM1, 4:ALM2, 5:ALM3, 6:ALM4, 7:CONF
	static char mode = 0;	// 0:DISPLAY, other:EDIT
	static char year, month, day, hour, min;

	if(mode == 0){
		if( key & 0x02 && screen < 7) screen++;
		if( key & 0x04 && screen > 0) screen--;
		if( key & 0x10 && screen != 0 && screen != 7){
			year  = g_RtcDateData.year;
			month = g_RtcDateData.month;
			day   = g_RtcDateData.day;
			hour  = g_RtcTimeData.hour;
			min   = g_RtcTimeData.min;
			mode  = 1;
		}
		switch(screen){
			case 0: dispDateTime(); break;
			case 1: dispDate(); break;
			case 2: dispTime(); break;
			case 3: dispAlarm(0, key); break;
			case 4: dispAlarm(1, key); break;
			case 5: dispAlarm(2, key); break;
			case 6: dispAlarm(3, key); break;
			case 7: dispSignalMode(key); break;
			default: break;
		}
	}
	else{
		if( key & 0x10 ){
			if(screen==1) writeRtcDate( year, month, day );
			if(screen==2) writeRtcTime( hour, min, 0 );
			if(screen>2 && screen<7) saveAlarmInfo();
			mode = 0;
		}
		switch(screen){
			case 0: break;
			case 1: editDate(key, &year, &month, &day); break;
			case 2: editTime(key, &hour, &min); break;
			case 3: editTime(key, &(g_AlarmInfo[0].hour), &(g_AlarmInfo[0].min) ); break;
			case 4: editTime(key, &(g_AlarmInfo[1].hour), &(g_AlarmInfo[1].min) ); break;
			case 5: editTime(key, &(g_AlarmInfo[2].hour), &(g_AlarmInfo[2].min) ); break;
			case 6: editTime(key, &(g_AlarmInfo[3].hour), &(g_AlarmInfo[3].min) ); break;
			case 7: break;
			default: break;
		}
	}
}

/******************************************************************************
	Main
******************************************************************************/
int main(void)
{
	char timerCounterPrev = 0;
	char key;

	_delay_ms(100);

	initGPIO();
	initUSART();
	initTWI();
	initLCD();
	initTimer();
	initPrint();
	loadAlarmInfo();
	soundSleep();

	while(1)
	{
		while(timerCounterPrev == timerCounter100ms);
		timerCounterPrev = timerCounter100ms;

		readRtcTime( &g_RtcTimeData );
		readRtcDate( &g_RtcDateData );

		key = readKey();
		screenManagement(key);
		voiceControl(key);
	}
}
